;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x30f87db977c27aa1024b28096f04c522 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x127e08513fb9bd46bfc2118c91295592 . "./code.cg2js/cg-printing.fasl")
(#x280f2532caff465a9cb12670c4fab373 . "./code.cg2js/cg-caret.fasl")
(#x457a32c8145ca407e5c9dc80b999850b . "./code.cg2js/cg-item-list.fasl")
(#x57b33e9ab9b013f7de8820e355795cb6 . "./code.cg2js/cg-progress-indicator.fasl")
(#x71810b75feda799605e029fec30653dd . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x7c8aca8ed1802f82fa104a9e5a2aebf9 . "./code.cg2js/cg-outline.fasl")
(#x42c2796a091cf0943b4626416726fdcc . "./code.cg2js/cg-palette.fasl")
(#x64d6c26bbd1701b7b5c535709417bb2c . "./code.cg2js/cg-selected-object.fasl")
(#x26d6d49eeb91bf454bfcc949b121d7cd . "./code.cg2js/cg-status-bar.fasl")
(#x00181036303b099ce14455d044a2806c . "./code.cg2js/cg-wrap-string.fasl")
(#x9e672f6dfef93b6f4ead467736168b89 . "./code.cg2js/cg-fill-texture.fasl")
(#x9ee3a357c64a8b7bfc7e9169737d28e0 . "./code.cg2js/cg-layout.fasl")
(#xf7b1b1a13517bbb9592c367431248b4c . "./code.cg2js/cg-timer.fasl")
(#xaec16f86c89a254cfdeb870d226bb848 . "./code.cg2js/cg-dialog-item.fasl")
(#xc1c01af9af06b01e5dd19ccf61d49b97 . "./code.cg2js/cg-button.fasl")
(#x2b22ff89d4ded62d915a3af21e857cd7 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xab7caef74cb734b71970288d97b61362 . "./code.cg2js/cg-tray.fasl")
(#x5312ce34ab3efc630fda772479f168cf . "./code.cg2js/cg-plot-widget.fasl")
(#x1557aead72cdffedf6bcbf0059dadeeb . "./code.cg2js/cg-pixmap.fasl")
(#xdfa0123c472082a3afa76f8c83931b14 . "./code.cg2js/cg-font-dialog.fasl")
(#xd964f43a14e0a50883ad2d312cbc6130 . "./code.cg2js/cg-os-widget.fasl")
(#xb942f1213ebee09522fe881a3fc9bf8d . "./code.cg2js/cg-text-or-combo.fasl")
(#x00b5c7d3baa58a1f464a150ef5770429 . "./code.cg2js/cg-trackbar.fasl")
(#xf8509b730369cf40306027380158b6d0 . "./code.cg2js/cg-up-down-control.fasl")
(#x214522b3c6a1cce55a3e3377f00f59a2 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x33e81c0a3526100548607b296778deac . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x9421db56e7da73bde4d60669e1dd7cc2 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x0e5c6961a32a4ca702953b3f26c6d488 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xe1021e910f145cbe8dcd8ce58ad837bc . "./code.cg2js/cg-chart-widget.fasl")
(#xd21db8b160460626633f9e6f66d8d4b1 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xda0681dd0fb8c2768a878ae60854eff2 . "./code.cg2js/cg-group-box.fasl")
(#x0a3ea081a5ac9c08245e1c169c9b9b4b . "./code.cg2js/cg-static-text.fasl")
(#x0826472043af332b1257bff8f62e0516 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x533cb1b59c1d5850801afb2fd5147d39 . "./code.cg2js/cg-color-dialog.fasl")
(#x1d80cbb3f72ed87342d175996a98699a . "./code.cg2js/cg-intersect.fasl")
(#xbe63aaff9c2ef16e9c01e42fb6fac402 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x3eb84d21a564b05dd187330e9f96977b . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x73570b234f7ad3c6f67c490ff3b560aa . "./code.cg2js/cg-html-dialog.fasl")
(#xe34fc0b7fea45aff6550eba801f087c8 . "./code.cg2js/cg-lettered-menu.fasl")
(#xe2eca24c61bf7aa276d6e4ea21a24feb . "./code.cg2js/cg-class-grid.fasl")
(#x5bbb0db47c7de52ecff23d5223cd7b7b . "./code.cg2js/cg-pixmap-printing.fasl")
(#xa1792d17415872e662f6c461a63cdf07 . "./code.cg2js/cg-calendar.fasl")
(#x91b32d95fe0a0ccc6db241681a07ab29 . "./code.cg2js/cg-editable-text.fasl")
(#x109799afb2934040bc32a6d783bd86c5 . "./code.cg2js/cg-property.fasl")
(#xb70a500ee87c1ccba69c84f4ca8b0bb7 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xb21f0c6f0f89a1f6f75685cb1a6ac620 . "./code.cg2js/cg-grid-widget.fasl")
(#xea35eec3c63e040f81c2f9c8206073c6 . "./code.cg2js/cg-paren-matching.fasl")
(#x850b4032d6613f02680fbd7d14b5aa07 . "./code.cg2js/cg-dev.fasl")
(#x514e47de525d36e04da329b370cc5378 . "./code.cg2js/cg-drawable.fasl")
(#x79f0c95d3b97ea08f6279c84a16272f4 . "./code.cg2js/cg-curve.fasl")
(#x10341a959a97e1f752212d7a566777db . "./code.cg2js/cg-lisp-widget.fasl")
(#x44e8145744e914115f7f326001a19625 . "./code.cg2js/cg-dde-utils.fasl")
(#xa869239ab83bc00b159dd94b1433ef3a . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x4d714e81a551be64ba1e58b1b785d4e3 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xe69dc429bdc72d12d5d8da1d368331b4 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x75c3dd70619cb4b42ee77aaa1d85eb01 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xce618df37b8cf2ebee8e718f7a7f45b3 . "./code.cg2js/cg-lamp.fasl")
(#x03932b985abd92f7815c96affb3fe79f . "./code.cg2js/cg-common-control.fasl")
(#xcbf3783c11ddcb93c1b35187634ae000 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x886d9c033da4089a0dc8e68d50347c93 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe08f7d1e425c9f437f424a6f740a6367 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x7c521918e293f42c7a3593a714b4cc60 . "./code.cg2js/cg-project-window.fasl")
(#x5b28b495e9c2a180b9b9aeb5d380c8e2 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xf36f780b8e5f61a9e0a3d9e9b51de49a . "./code.cg2js/cg-class-support.fasl")
(#x537ff645cb5d1fe84649b56f8d57e39e . "./code.cg2js/cg-multi-picture-button.fasl")
(#x8d18bede651708eff5f53ec1270b0b20 . "./code.cg2js/cg-yes-no-list.fasl")
(#x467d5228309d81ec73885e4777848111 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xa9bd77c25ce290330158f807f6cc86c0 . "./code.cg2js/cg-string-dialog.fasl")
(#xbf193fb9d6bacf81799b31fde817858c . "./code.cg2js/cg-menu-tooltip.fasl")
(#xf5dad39a5b2c76895ad0b434fd3f922e . "./code.cg2js/cg-dropping-outline.fasl")
(#xde0de35ce1efc324fba0ec9314a899f5 . "./code.cg2js/cg.fasl")
(#xb10fc78073b37bb2263358fc1311e931 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xd41d63ed1a2b7f5cfff822c0f72e8125 . "./code.cg2js/cg-scaling-stream.fasl")
(#xde628d4debed4c91bf445a26e9226289 . "./code.cg2js/cg-yes-no-string.fasl")
(#xfc233e493406350e85a003c5182d7e7d . "./code.cg2js/cg-icon-pixmap.fasl")
(#x908f858b1110993c115bc5f7fd19d915 . "./code.cg2js/cg-web-browser.fasl")
(#x2e1a4ea545f2c11a27f1b6b73338586d . "./code.cg2js/cg-clipboard-stack.fasl")
(#x279af2570386f2fe3dde01077c5d9f2b . "./code.cg2js/cg-drag-and-drop.fasl")
(#x601eba05cd6d35e828a76cbd00a3cb58 . "./code.cg2js/cg-message-dialog.fasl")
(#xba7759f3e4257ba3ed5ec432dee83da7 . "./code.cg2js/cg-comtab.fasl")
(#x8cce0a9ccc2eb08b5769f1a1dab06dac . "./code.cg2js/cg-base.fasl")
(#xba835f96e5a94e809eb8ba77f9ff6748 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xa2d18f0d4e506072993ec293ac27d95f . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x6d9559b9556be875e4fe16233671ef4e . "./code.cg2js/cg-text-edit-pane.fasl")
(#xe157cb6115f6ba9d01af9968cfa0a846 . "./code.cg2js/cg-utility-dialog.fasl")
(#x54c4e2cb9d302f2cbf444983935c1fcc . "./code.cg2js/cg-object-editor.fasl")
(#x860f62ef3bc65a33455538a26089c137 . "./code.cg2js/cg-edit-in-place.fasl")
(#x253ed633c65e12903dd70316c03a3b26 . "./code.cg2js/cg-radio-button.fasl")
(#xf760098aade24e156cdbe0fc63e4f237 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xca100753d1f6c07a40c6f47187fb0d6d . "./code.cg2js/cg-nodes.fasl")
(#x0fa046fb7d104c4367cad406d439c132 . "./code.cg2js/cg-get-position.fasl")
(#x8bacd9f223aa646d6b82b03c6f530f68 . "./code.cg2js/cg-directory-dialog.fasl")
(#x87989af7f1cccd1adb290c328791ebea . "./code.cg2js/cg-lisp-text.fasl")
(#x0323a334d89fffb7a0d65608ea2088b4 . "./code.cg2js/cg-combo-box.fasl")
(#xdb9e39778992a2c1b72d5a2ca8ff3570 . "./code.cg2js/cg-text-widget.fasl")
(#xe74266fcd57b1cba0c3cf120c8e5c713 . "./code.cg2js/cg-list-view.fasl")
(#x38b775fff4977619a8f298fffaf3a4a4 . "./code.cg2js/cg-header-control.fasl")
(#x3edf04b135dc9eb496b26841ac491221 . "./code.cg2js/cg-graphics-context.fasl")
(#x54451aba6edeed54ca7431e8fb5b80b4 . "./code.cg2js/cg-get-pixmap.fasl")
(#x4300f6fc17040a75ea9c8b5572a40986 . "./code.cg2js/cg-menu.fasl")
(#x1712e949beb996c0e29811a048a7334b . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x81e194dcb8c8d298f65f4248cfe5af95 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x6ebe21bfffd0955a72a535a7cd87f026 . "./code.cg2js/cg-tooltip.fasl")
(#xed6c09dff48cfb981ddb6961effcf086 . "./code.cg2js/cg-html-widget.fasl")
(#xa8b4cac14dda1ff52a70d08c048ab6ae . "./code.cg2js/cg-choice-list.fasl")
(#x04b099f9a800b6b6e26e593092f9b201 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xb6ec0e24614607545698032d703b91f6 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x70aa970fc9a59370da0253b9fbda5f58 . "./code.cg2js/cg-icon.fasl")
(#x7b75b46a71cb2ac1c8eb175d4d8acd49 . "./code.cg2js/cg-acache.fasl")
(#x25e68f8ed57837688fa806177070ee6f . "./code.cg2js/cg-template-string.fasl")
(#x2320cb3f902f3247f2e527bfc8d6a0bf . "./code.cg2js/cg-bitmap-pane.fasl")
(#xdc3b041dc9af51f8e7a0eba5ab14b153 . "./code.cg2js/cg-toggling-widget.fasl")
(#xd23242380e75f2be3cf6e3ce1cf50659 . "./code.cg2js/cg-tab-control.fasl")
(#x776c4a1d88caff34ef02f222035de924 . "./code.cg2js/cg-split-bar.fasl")
(#x6225e7c31c3bff7d14a14cf326df7790 . "./code.cg2js/cg-picture-widget.fasl")
(#x8752f354cd93a55a968f23397afd442f . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x8c8b4140ada1d2e17ba19abbe0249d61 . "./code.cg2js/cg-choose-printer.fasl")
(#xcd20d6748ae1db5b51c54756abe0b4f4 . "./code.cg2js/cg-os-window.fasl")
(#x51ef88caaae6c5d292caf57174aa33b5 . "./code.cg2js/cg-scroll-bar.fasl")
(#x13158f4e73f822371382764ad75ec62b . "./code.cg2js/cg-clipboard.fasl")
(#x91a8527c1cb488f2a150bf6cbd313026 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xbfaf20ddeab532ef4d01fa46d403b1e8 . "./code.cg2js/cg-hotspot.fasl")
(#x46472250d2cba078751adaa592175a57 . "./code.cg2js/cg-rich-edit.fasl")
(#x5cea742678d8017b915cb63c5597f372 . "./code.cg2js/cg-check-box.fasl")
(#xc808acb839fc5d27c8dd7fdeb43a7eb8 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xc065131d26b49df48a892f07d9d35ba7 . "./code.cg2js/cg-file-dialog.fasl")
(#x5b8d96b7c2756b4a43f74c9e2ad67a09 . "./code.cg2js/cg-toolbar.fasl")
(#x08bea9c2ad952dbb956109c438fdd7fd . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x181cfd021fabd98a76454c8fd7f76fb3 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x0cdf2a8942b1f4736e9b05d8e8c0c89e . "./code.cg2js/cg-shortcut-menu.fasl")
(#x3af320f868f793a3b83863763cc72929 . "./code.idejs/ide-project.fasl")
(#xdb7f85beefd41c1c9cdd457b9bf23a1c . "./code.idejs/ide-intro.fasl")
(#x702a16feebb6bf34e3417fae66d2eb93 . "./code.idejs/ide-find-in-files.fasl")
(#x4a4fe6d9d237d26be489bf73c214f3f4 . "./code.idejs/ide-grapher.fasl")
(#x43fef401c138aa2a3e0e87558d5c85a1 . "./code.idejs/ide-inspector.fasl")
(#xc15e1db9a22c9e27927c955c6a6ab2ea . "./code.idejs/ide-editor.fasl")
(#x6fd33c7be8548e8269e975246e551c3e . "./code.idejs/ide-window-list.fasl")
(#xe8c7d2b9cfc05c7080c4878b02091988 . "./code.idejs/ide-process-browser.fasl")
(#x7018dab7ceb00765390ccb52477bc6fe . "./code.idejs/ide-patch-dialog.fasl")
(#x063bcb2dd44bfd177db1e23264097e12 . "./code.idejs/ide-class-grapher.fasl")
(#xc4e6d5b11a5ab1e7fc0b7eb342e9f794 . "./code.idejs/ide-base.fasl")
(#xf412a828bb9c9693edd39d8589a6a133 . "./code.idejs/ide.fasl")
(#xf77dfec34be75c9ef15df2c9baaed019 . "./code.idejs/ide-trace.fasl")
(#xb9793cedbd094024f87c1220d6ed1517 . "./code.idejs/ide-form.fasl")
(#x9f89c096e9f59658a13e9f32f0f1c643 . "./code.idejs/ide-find-definitions.fasl")
(#x1e4df00d4f7d07386322f0343fdd2f72 . "./code.idejs/ide-apropos.fasl")
(#x501e8ec2934c422f291ac368a4fbcb6d . "./code.idejs/ide-class-browser.fasl")
(#xdf795230ad3a120c25804c3fa31cc63d . "./code.idejs/ide-navigator.fasl")
(#x3d115baf169f24c484240d2c767f2292 . "./code.idejs/ide-clipboard.fasl")
(#xaabb875a9d71b23b79d40e3bfa56fa81 . "./code.idejs/ide-acache.fasl")
(#x48f463744c0403196deba34e7d7ec736 . "./code.idejs/ide-keyshort.fasl")
(#x173c7777f4b3f49fab3973bb1950c67b . "./code.idejs/ide-package-list.fasl")
(#x2e5e0632963fdcd2d4e26e4f0e9d5356 . "./code.idejs/ide-dev.fasl")
(#xeb5b70ef6a4ced03becd92b82d3b0061 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
